﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace CameraPreviewTest {
    public class FrameBuffer {
        // 画像を格納するバッファ
        public int[] Buffer { get; set; }

        // バッファに格納する画像の幅
        public int Width { get; set; }

        // バッファに格納する画像の高さ
        public int Height { get; set; }

        public FrameBuffer(int width, int height) {
            Buffer = new int[width * height];
            Width = width;
            Height = height;
        }

        /// <summary>
        /// 画像バッファをセピア調化する
        /// </summary>
        public void EffectSepiaTone() {
            for (int i = 0; i < Buffer.Length; i++) {
                int pixel = Buffer[i];

                // RGB値を取り出す
                byte r = Convert.ToByte((pixel >> 16) & 0xff);
                byte g = Convert.ToByte((pixel >> 8) & 0xff);
                byte b = Convert.ToByte(pixel & 0xff);

                // 平均値を求めてグレースケール化する
                byte avg = (byte)((r + g + b) / 3);

                // セピアに近づける為、計算を行う
                r = (byte)(avg * 1);
                g = (byte)(avg * 0.7);
                b = (byte)(avg * 0.4);

                Buffer[i] = 255 << 24 | r << 16 | g << 8 | b;
            }
        }

        /// <summary>
        /// 画像バッファをグレイスケール化する
        /// </summary>
        public void EffectGrayscale() {

            for (int i = 0; i < Buffer.Length; i++) {
                int pixel = Buffer[i];

                // RGB値を取り出す
                byte r = Convert.ToByte((pixel >> 16) & 0xff);
                byte g = Convert.ToByte((pixel >> 8) & 0xff);
                byte b = Convert.ToByte(pixel & 0xff);

                // 単純に平均値を取る
                byte avg = (byte)((r + g + b) / 3);

                Buffer[i] = 255 << 24 | avg << 16 | avg << 8 | avg;
            }
        }

        /// <summary>
        /// 画像バッファをネガポジ反転させる
        /// </summary>
        public void EffectNegative() {
            for (int i = 0; i < Buffer.Length; i++) {
                int pixel = Buffer[i];
                // 各RGB要素のビットを反転させる
                Buffer[i] = pixel ^ 0x00ffffff;
            }
        }

        public enum ColorEffectType {
            Red,
            Green,
            Blue
        }

        /// <summary>
        /// 画像バッファにカラーエフェクトを掛ける
        /// </summary>
        public void EffectColor(ColorEffectType effectType) {
            // 各RGB要素のいずれかの要素だけ有効にする
            if (effectType == ColorEffectType.Red) {
                for (int i = 0; i < Buffer.Length; i++)
                    Buffer[i] &= 0xff0000;
            } else if (effectType == ColorEffectType.Green) {
                for (int i = 0; i < Buffer.Length; i++)
                    Buffer[i] &= 0x00ff00;
            } else if (effectType == ColorEffectType.Blue) {
                for (int i = 0; i < Buffer.Length; i++)
                    Buffer[i] &= 0x0000ff;
            }
        }
    }
}
